from typing import Dict

import os
import requests


class API:
    '''
        API Class handles all HTTP Requests

        Attributes:
            base_url(string): REST API base URL for Sinequa instance
            access_token(string): token for Sinequa authentication
    '''

    def __init__(self, access_token: str, base_url: str) -> None:
        if not access_token or not base_url:
            raise ValueError("access_token and base_url must not be empty")

        self.access_token = access_token
        self.base_url = base_url

    def _get_headers(self) -> Dict:
        headers = {
            "Authorization": f"Bearer {self.access_token}"
        }
        return headers

    def _get_url(self, endpoint) -> str:
        return os.path.join(self.base_url, endpoint)

    def get(self, endpoint) -> Dict:
        """
            This method handles GET method.
        """
        session = requests.Session()
        resp = session.get(self._get_url(endpoint=endpoint),
                           headers=self._get_headers())
        session.close
        return resp.json()

    def post(self, endpoint, payload) -> Dict:
        """
            This method handles POST method.
        """
        session = requests.Session()
        resp = session.post(self._get_url(endpoint=endpoint),
                            headers=self._get_headers(), json=payload)
        return resp.json()



from __future__ import annotations
from typing import Optional, List, Dict

from pynequa.api import API
from pynequa.models import QueryParams, TreeParams


class Sinequa(API):
    '''
        Sinequa API Client for Python

        Attributes:
            access_token(str): authentication token for Sinequa
            base_url(str): base URL for hosted Sinequa instance
            app_name(str): name of Sinequa app
            query_name(str): name of search query web service
    '''

    def __init__(
            self,
            access_token: str,
            base_url: str,
            app_name: str = "vanilla-search",
            query_name: str = "query",
    ) -> None:
        super().__init__(access_token=access_token, base_url=base_url)
        self.app_name = app_name  # name of application
        # name of search query web service
        self.query_name = query_name

    @classmethod
    def from_config(cls, cfg: Dict) -> Sinequa:
        '''
        This method creates an instance of Sinequa class from a configuration
        dictionary.

        Args:
            cfg (Dict): A dictionary containing configuration parameters.
                It should include the following keys:
                - "access_token": A valid access token for authentication.
                - "base_url": The base URL for the Sinequa API.
                - "app_name": Name of the Sinequa application.
                - "query_name": Name of the search query service.

        Returns:
            Sinequa: An instance of Sinequa class initialized with given configuration
        '''
        return cls(
            access_token=cfg["access_token"],
            base_url=cfg["base_url"],
            app_name=cfg["app_name"],
            query_name=cfg["query_name"],
        )

    @staticmethod
    def _prepare_kwargs(payload: Dict, kwargs: Dict) -> Dict:
        for key, value in kwargs.items():
            payload[key] = value
        return payload

    def search_app(self, pre_login: bool = False, mode: str = "debug") -> Dict:
        '''
            This method retrieves SBA configuration before and after login.

            Args:
                pre_login(bool): false by default.
                mode(str): debug by default (debug|release)
        '''
        endpoint = "search.app"
        payload = {
            "app": self.app_name,
            "preLogin": pre_login,
            "mode": mode,
        }
        return self.post(endpoint=endpoint, payload=payload)

    def search_dataset(self, parameters: Optional[Dict],
                       datasets: Optional[List[str]]) -> Dict:
        '''
        This method retrieves datasets through SQL queries. The response is a
        list of available datasets with their respective names and descriptions.

        Args:
            parameters(dict): dictionary of parameters that can be used in SQL fragments
            datasets(list): list of datasets
        Returns:
            Dict: search dataset response

        '''
        endpoint = "search.dataset"
        payload = {}
        if parameters is not None:
            payload["parameters"] = parameters

        if len(datasets) > 0:
            payload["datasets"] = datasets

        return self.post(endpoint=endpoint, payload=payload)

    def search_query(self, query_params: QueryParams) -> Dict:
        '''
        This method performs search query.

        Args:
            query_params(QueryParams): query parameters as defined in QueryParams class

        Returns:
            Dict: response data of this request
        '''
        endpoint = "search.query"

        payload = {
            "app": self.app_name,
            "query": query_params.generate_payload(
                query_name=self.query_name)
        }

        return self.post(endpoint=endpoint, payload=payload)

    def query_intent(self, intent_text: str) -> Dict:
        '''
        This method evaluates SBA query intents from query intent sets.

        Args:
            intent_text(str): indicates the text that should trigger query intent

        Returns:
            Dict: Query Intent response

        '''
        endpoint = "queryintent"

        query_parameters = {
            "name": self.query_name,
            "text": intent_text
        }
        payload = {
            "app": self.app_name,
            "name": self.query_name,
            "query": query_parameters,
        }

        return self.post(endpoint=endpoint, payload=payload)

    def search_profile(self, profile_name: str, query_params: QueryParams,
                       response_type: str = "SearchCursor") -> Dict:
        '''
        This method searches for Sienequa profile.

        Args:
            profile_name(str): profile name
            response_type(str): default will be SearchCursor
            query_params(QueryParams): will carry  query parameters in payload

        Returns:
            Dict: response data for Sinequa profile search.
        '''
        endpoint = "search.profile"

        payload = {
            "profile": profile_name,
            "responsetype": response_type,
            "query": query_params.generate_payload(query_name=self.query_name),
        }

        return self.post(endpoint=endpoint, payload=payload)

    def search_user_settings(self, action: str = "load",
                             user_settings: Dict = {}) -> Dict:
        '''
        This method provides user settings

        Args:
            app_name(str): name of application for which user setting should be handled
            action(str): search action (load|save|patch)
            user_settings(Dict): user settings to be saved or patched (see official
            documentation for more info)

        Returns:
            Dict: search response based upon action
        '''
        endpoint = "search.usersettings"
        payload = {
            "app": self.app_name,
            "action": action,
            "userSettings": user_settings,
        }

        return self.post(endpoint=endpoint, payload=payload)

    def search_preview(self,   query_params: QueryParams, action: str = "get",
                       origin: str = "", id: str = "") -> Dict:
        '''
        This method retrieves preview data for a product.

        Args:
            query_params(QueryParams): query params with text to be searched
            action(str): retrieves a preview object
            origin(str): server address of the SBA in browser used to load CSS
            id(str): id of document for which to retrieve the preview

        Returns:
            Dict: response for for Search Preview
        '''
        endpoint = "search.preview"

        payload = {
            "app": self.app_name,
            "action": action,
            "id": id,
            "origin": origin,
            "query": query_params.generate_payload(
                query_name=self.query_name)
        }

        return self.post(endpoint=endpoint, payload=payload)

    def search_query_export(self, web_service: str, type: str, format: str,
                            name: str, max_count: int) -> Dict:
        '''
        This method allows exporting of search results in different formats.

        Args:
            web_service(str): name of query export web service
            type(str): type of export to perform
            format(str): output format of export
            name(str): name of saved query to be exported (optional)
            max_count(int): maximum of number of documents to include in export(optiona)

        Returns:
            Dict: response for search export
        '''
        endpoint = "search.queryexport"
        payload = {
            "app": self.app_name,
            "webService": web_service,
            "type": type,
            "format": format,
        }

        if name != "":
            payload["name"]: name

        if max_count > 0:
            payload["maxCount"] = max_count

        return self.post(endpoint=endpoint, payload=payload)

    def search_recent_queries(self, action: str = "load") -> Dict:
        '''
        This method retrieves the current recent queries.

        Args:
            action(str): Retrieve recent queries, default=load

        Returns:
            Dict: response for recent queries
        '''
        endpoint = "search.recentqueries"

        payload = {
            "app": self.app_name,
            "action": action,
        }
        return self.post(endpoint=endpoint, payload=payload)

    def search_similardocuments(self, source_doc_id: str,
                                query_params: QueryParams) -> Dict:
        '''
        This method retrieves similar documents to a given document

        Args:
            source_doc_id(str): identifier of document for which to retrieve similar documents
            query_params(QueryParams): query params

        Returns:
            Dict: search response for similar documents
        '''
        endpoint = "search.similardocuments"

        payload = {
            "app": self.app_name,
            "sourceDocumentId": source_doc_id,
            "query": query_params.generate_payload(
                query_name=self.query_name)
        }

        return self.post(endpoint=endpoint, payload=payload)

    def search_query_links(self, web_sevice: str, query_params: QueryParams) -> Dict:
        '''
        This method retrieves sponsored links for the passed query.

        Args:
            web_service(str): name of corresponding sponsored links web service
        Returns:
            Dict: response for query links search

        '''
        endpoint = "search.querylinks"
        payload = {
            "webService": web_sevice,
            "query": query_params.generate_payload(
                query_name=self.query_name)
        }
        return self.post(endpoint=endpoint, payload=payload)

    def search_ratings(self, action: str, docid: str, ratings_column: str,
                       avg_column: str, ratings_distribution: List[str], rating: int,
                       update_doc_weight: bool) -> Dict:
        '''
        This method makes it possible to get, set and delete ratings associated with a
        document.

        Args:
            action(str): get|set|delete
            docid(str): document id
            ratings_column(str): name of column in which to store rating
            avg_column(str): name of column to store average rating
            ratings_distribution(List[str]): array of possible ratings
            rating(int): sets the action parameter (optional)
            update_doc_weight(bool): indicates whether to update the doc weight
                                    according to rating (optional)
        Returns:
            Dict: response for ratings search
        '''
        endpoint = "search.ratings"
        payload = {
            "action": action,
            "docid": docid,
            "ratingsColumn": ratings_column,
            "averageColumn": avg_column,
            "ratingsDistribution": ratings_distribution,
            "updateDocWeight": update_doc_weight,
        }

        if len(rating) > 0:
            payload["rating"] = rating

        return self.post(endpoint=endpoint, payload=payload)

    def search_profile_subtree(self, profile: str, query_params: QueryParams,
                               tree_params: TreeParams) -> Dict:
        '''
        This method allows to select a subtree.

        Args:
            profile(str): profile name
            query_params(QueryParams): search parameters
            tree_params(TreeParams): defines the sub-tree to retrieve

        Returns:
            Dict: returns subtree profile response
        '''
        endpoint = "search.profile.subtree"
        payload = {
            "profile": profile,
            "query": query_params.generate_payload(
                query_name=self.query_name),
            "tree": tree_params.generate_payload()
        }
        return self.post(endpoint=endpoint, payload=payload)

    def search_alerts(self):
        '''
        '''
        endpoint = "search.alerts"
        pass

    def search_baskets(self):
        '''
        '''
        endpoint = "search.baskets"
        pass

    def search_labels(self):
        '''
        '''
        endpoint = "search.labels"
        pass

    def search_saved_queries(self):
        '''
        '''
        endpoint = "search.savedQueries"
        pass

    def search_suggest(self):
        '''
        '''
        endpoint = "search.suggest"
        pass

    def search_custom(self):
        '''
        '''
        endpoint = "search.custom"
        pass

    def suggest_field(self):
        '''
        '''
        endpoint = "suggestField"
        pass

    def engine_sql(self, sql: str, max_rows: int = 1000) -> Dict:
        '''
        This method runs classic SQL query on the database engine.

        Args:
            sql (str): Query to send
            max_rows (int): Maximum number of rows a query can return (default: 1000)
        Returns:
            Dict: response for sql query
        '''
        endpoint = "engine.sql"

        payload = {
            "sql": sql,
            "maxRows": 1000,
            "pretty": True,
            "output": "json"
        }

        return self.post(endpoint=endpoint, payload=payload)



from pynequa import Sinequa
from pynequa.models import QueryParams
import os
import unittest

base_url = os.environ.get("SINEQUA_BASE_URL")
app_name = "vanilla-search"
query_name = "query"


class TestSinequaSearchQuery(unittest.TestCase):
    """
    A unittest class for testing Sinequa search queries.

    This class contains two test methods that evaluate the behavior of Sinequa
    search queries under different authentication scenarios.

    Test Methods:
        - `test_search_query_without_auth`: Tests a search query without
          authentication. It creates a Sinequa instance using provided
          configuration, sends a search query, and checks the error code in
          the response.

        - `test_search_query_with_auth`: Tests a search query with
          authentication. It retrieves an access token from environment
          variables, creates a Sinequa instance using provided configuration,
          sends a search query, and checks the method result in the response.

    """

    def test_search_query_without_auth(self):
        """
        Test a search query without authentication.

        This test case creates a Sinequa instance with the given configuration,
        sets up query parameters, sends a search query, and asserts that the
        expected error code is returned in the response.
        """

        config = {
            "base_url": base_url,
            "app_name": app_name,
            "access_token": "pass",
            "query_name": query_name
        }
        query_params = QueryParams()
        query_params.search_text = "NASA"

        sinequa = Sinequa.from_config(config)
        resp = sinequa.search_query(query_params=query_params)
        self.assertEqual(resp["ErrorCode"], 6)

    def test_search_query_with_auth(self):
        """
        Test a search query with authentication.

        This test case retrieves an access token from environment variables,
        creates a Sinequa instance with the given configuration, sets up query
        parameters, sends a search query, and asserts that the method result in
        the response is as expected.
        """
        access_token = os.environ.get("SINEQUA_ACCESS_TOKEN")
        config = {
            "base_url": base_url,
            "access_token": access_token,
            "app_name": "vanilla-search",
            "query_name": "query"
        }
        query_params = QueryParams()
        query_params.search_text = "NASA"

        sinequa = Sinequa.from_config(config)
        resp = sinequa.search_query(query_params=query_params)
        self.assertEqual(resp["methodresult"], "ok")


if __name__ == '__main__':
    unittest.main()



import unittest
from unittest.mock import patch
from pynequa.api import API
import logging

mock_config = {
    "access_token": "hzyasdfewr.asdfdsafdsaf.zsdfsdafdsafdsafdsafdsafdsafdsafdsafdsafdsafdsafdsafds",
    "base_url": "https://api.sinequa.com"
}

sample_response = {"msg": "success"}


class TestAPI(unittest.TestCase):

    @patch('pynequa.api.API.get')
    def test_get(self, mock_get):
        mock_get.return_value.status_code = 200
        mock_get.return_value.json.return_value = sample_response

        api = API(mock_config["access_token"], mock_config["base_url"])
        resp = api.get("search.query")

        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json(), sample_response)

    @patch('pynequa.api.API.post')
    def test_post(self, mock_post):
        mock_post.return_value.status_code = 200
        mock_post.return_value.json.return_value = sample_response

        api = API(mock_config["access_token"], mock_config["base_url"])
        endpoint = "search.query"
        payload = {
            "app": "sba",
            "query": {
                "query_name": "sba-angular",
                "text": "When was Landsat-9 launched?"
            }
        }
        resp = api.post(endpoint=endpoint, payload=payload)

        self.assertEqual(resp.status_code, 200)
        self.assertEqual(resp.json(), sample_response)


if __name__ == '__main__':
    unittest.main()



from typing import Dict, List, Optional
from abc import abstractmethod, ABC
from dataclasses import dataclass, field
from loguru import logger


class AbstractParams(ABC):
    """
    Abstract base class for all Sinequa models.
    """

    @abstractmethod
    def generate_payload(self, **kwargs) -> Dict:
        """
        This is abstract method for AbstractParams.
        Every child class should implement this method.
        """
        raise NotImplementedError()


@dataclass
class TreeParams(AbstractParams):
    """
    Represents the parameters for configuring a tree parameters.

    Attributes:
        box (str): The name of the relevant tree navigation box (required).
        column (str): The name of the index column associated with the
                        navigation box (required).
        op (str, optional): The relational operator. Default is 'eq'.
            Possible values: '=', '!=', '<', '<=', '>=', '>', 'between', 'not between'.
        value (str): The filter value (required).
    """
    box: str = ""
    column: str = ""
    op: str = ""
    value: str = ""

    def generate_payload(self, **kwargs) -> Dict:
        """
        This method generates payload for
        TreeParams.
        """
        return {
            "box": self.box,
            "column": self.column,
            "op": self.op,
            "value": self.value,
        }


@dataclass
class SelectParams(AbstractParams):
    expression: str = ""
    facet: str = ""

    def generate_payload(self, **kwargs) -> Dict:
        """
        This method generates payload for
        SelectParams.
        """
        return {
            "expression": self.expression,
            "facet": self.facet,
        }


@dataclass
class OpenParams(AbstractParams):
    expression: str = ""
    facet: str = ""

    def generate_payload(self, **kwargs) -> Dict:
        """
        This method generates payload for
        OpenParams.
        """
        return {
            "expression": self.expression,
            "facet": self.facet,
        }


@dataclass
class AdvancedParams(AbstractParams):
    col_name: str = ""
    col_value: str = None
    value:  str or int = None
    operator: str = None
    debug: bool = False

    def generate_payload(self, **kwargs) -> Dict:
        """
        This method generates payload for
        AdvancedParams.
        """
        payload = {
            self.col_name: self.col_value,
            "value": self.value,
            "operator": self.operator
        }

        if self.debug:
            logger.debug(payload)

        return payload


@dataclass
class QueryParams(AbstractParams):
    name: str = ""  # required
    action: Optional[str] = None
    search_text: str = ""  # required
    select_params: Optional[List[SelectParams]
                            ] = field(default_factory=lambda: [])
    additional_select_clause: Optional[str] = None
    additional_where_clause: Optional[str] = None
    open_params: Optional[List[OpenParams]] = field(default_factory=lambda: [])
    page: Optional[int] = 1
    page_size: Optional[int] = 10
    tab: Optional[str] = None
    scope: Optional[str] = None
    basket: Optional[str] = None
    is_first_page: Optional[bool] = False
    strict_refine: Optional[bool] = False
    global_relevance: Optional[int] = None
    question_language: Optional[str] = None
    question_default_language: Optional[str] = None
    spelling_correction_mode: Optional[str] = None
    spelling_correction_filter: Optional[str] = None
    document_weight: Optional[str] = None
    text_part_weights: Optional[str] = None
    relevance_transforms: Optional[str] = None
    remove_duplicates: Optional[bool] = False
    aggregations: Optional[List[str]] = field(default_factory=lambda: [])
    order_by: Optional[str] = None
    group_by: Optional[str] = None
    advanced: Optional[AdvancedParams] = None
    debug: bool = False

    def _prepare_query_args(self, query_name: str) -> Dict:
        params = {
            "name": query_name,
            "text": self.search_text,
            "isFirstpage": self.is_first_page,
            "strictRefine": self.strict_refine,
            "removeDuplicates": self.remove_duplicates,
        }

        if self.action is not None:
            params["action"] = self.action
        else:
            params["action"] = "search"

        if len(self.select_params) > 0:
            select_params = []
            for item in self.select_params:
                select_params.append(item.generate_payload())
            params["select"] = self.select_params

        if self.additional_select_clause is not None:
            params["additionalSelectClause"] = self.additional_select_clause

        if self.additional_where_clause is not None:
            params["additionalWhereClause"] = self.additional_where_clause

        if len(self.open_params) > 0:
            open_params = []
            for item in self.open_params:
                open_params.append(item.generate_payload())
            params["open"] = self.open_params

        if self.page is not None:
            params["page"] = self.page
        else:
            params["page"] = 1

        if self.page_size is not None:
            params["pageSize"] = self.page_size
        else:
            params["pageSize"] = self.page_size

        if self.tab is not None:
            params["tab"] = self.tab

        if self.scope is not None:
            params["scope"] = self.scope

        if self.basket is not None:
            params["basket"] = self.basket

        if self.global_relevance is not None:
            params["globalRelevance"] = self.global_relevance

        if self.question_language is not None:
            params["questionLanguage"] = self.question_language

        if self.question_default_language is not None:
            params["questionDefaultLanguage"] = self.question_default_language

        if self.spelling_correction_mode is not None:
            params["spellingCorrectionMode"] = self.spelling_correction_mode

        if self.spelling_correction_filter is not None:
            params["spellingCorrectionFilter"] = self.spelling_correction_filter

        if self.document_weight is not None:
            params["documentWeight"] = self.document_weight

        if self.text_part_weights is not None:
            params["textPartWeights"] = self.text_part_weights

        if self.relevance_transforms is not None:
            params["relevanceTransforms"] = self.relevance_transforms

        if len(self.aggregations) > 0:
            params["aggregations"] = self.aggregations

        if self.order_by is not None:
            params["orderBy"] = self.order_by

        if self.group_by is not None:
            params["groupBy"] = self.group_by

        if self.advanced is not None:
            params["advanced"] = self.advanced.generate_payload()

        return params

    def generate_payload(self, **kwargs) -> Dict:
        """
        This method generates payload for
        QueryParams.

        Args:
            query_name(str): Name of query service to query for
        """
        query_name = kwargs.get("query_name")
        payload = self._prepare_query_args(query_name)
        if self.debug:
            logger.debug(payload)

        return payload



from setuptools import setup, find_packages

VERSION = '0.2.0'
requirements = []
test_requirements = ['pytest>=3']

setup(
    name='pynequa',
    version=VERSION,
    author='Anish Bhusal',
    author_email='anish.bhusal@uah.edu',
    description='Pythonic Layer for Sinequa REST API',
    packages=find_packages(include=['pynequa', 'pynequa.*']),
    install_requires=requirements,
    test_suite='tests',
    tests_require=test_requirements,
)



from pynequa.models import QueryParams
import unittest
import logging


class TestQueryParams(unittest.TestCase):

    def test_query_params_payload(self):
        """
            Test if query params payload is correctly
            generated or not.
        """
        qp = QueryParams(
            name="query",
            search_text="What was Landsat-9 launched?"
        )

        payload = qp.generate_payload()
        logging.debug(payload)

        keys_which_must_be_in_payload = [
            "name",
            "text",
            "isFirstpage",
            "strictRefine",
            "removeDuplicates"
        ]

        for key in keys_which_must_be_in_payload:
            if key not in payload:
                self.assertEqual(key, "test", f"{key} is mising in payload")


if __name__ == '__main__':
    unittest.main()
